<?php

/**
 * @file
 * Administration functions for wow.module.
 */



/**
 * @} End of "wow-api-add"
 */

/**
 * @defgroup wow-api-services-predefined List of predefined services
 * @{
 * API to provide a list of predefined services.
 */

/**
 * Prepares the service code list for a select form item with only the unsupported ones.
 */
function _wow_prepare_predefined_list() {
  $services = wow_service_list();
  $predefined = _wow_get_predefined_list();
  foreach ($predefined as $key => $value) {
    if (isset($services[$key])) {
      unset($predefined[$key]);
      continue;
    }
    // Include native name in output, if possible
    if (count($value) > 3) {
      $tname = t($value[2]);
      $predefined[$key] = ($tname == $value[3]) ? $tname : "$tname ($value[3])";
    }
    $predefined[$key] = t($value[2]);
  }
  asort($predefined);
  return $predefined;
}


/**
 * Some of the supported services.
 */
function _wow_get_predefined_list() {
  include_once DRUPAL_ROOT . '/includes/iso.inc';
  $predefined = _locale_get_predefined_list();
  foreach (wow_api_info() as $region => $info) {
    foreach ($info->locales as $langcode => $locale) {
      $key = drupal_strtolower(str_replace('_', '-', $locale));
      if (array_key_exists($key, $predefined)) {
        $services[$key] = array_merge(array($region, $locale), $predefined[$key]);
      }
      else {
        $services[$langcode] = array_merge(array($region, $locale), $predefined[$langcode]);
      }
    }
  }
  return $services;
}

/**
 * @} End of "wow-api-services-predefined"
 */

/**
 * Prepare a batch to import data for all enabled modules in a given locale.
 *
 * @param string $langcode
 *   Language code to import data for.
 * @param string $finished
 *   Optional finished callback for the batch.
 * @param $force
 * TODO: implement this.
 *   An array of data resources to force refresh.
 */
function wow_batch_by_service($langcode, $finished = NULL, $force = array()) {
  $services = wow_service_list('language');
  // Collect all data endpoint to import for all enabled modules.
  $data = array();
  foreach (module_invoke_all('wow_data_resources') as $module => $resources) {
    foreach ($resources as $key => $resource) {
      // Collect all resources for all components, for a specific $langcode.
      // This allows for batch API to be able to import in small chunks of data.
      $last_fetched = @$services[$langcode]->lastFetched[$module][$key];
      $last_fetched = isset($last_fetched) ? $last_fetched : 0;
      if ($last_fetched + $resource['refresh threshold'] < REQUEST_TIME) {
        $resource += array(
          'data key' => $key,
          'base module' => $module,
          'base table' => $module . '_' . $key,
          'import callback' => $module . '_' . $key . '_import',
        );
        $data[] = $resource;
      }
    }
  }

  return _wow_batch_build($services[$langcode], $data, $finished);
}

/**
 * Build a wow service batch from an array of data resources.
 *
 * @param $service
 *   The service definition.
 * @param $resources
 *   Array of data resources to import.
 * @param $finished
 *   Optional finished callback for the batch.
 */
function _wow_batch_build($service, $resources, $finished = NULL) {
  if (count($resources)) {
    $operations = array();
    foreach ($resources as $resource) {
      // We call _wow_batch_import for every batch operation.
      $operations[] = array('_wow_batch_import', array($service, $resource));
    }
    $batch = array(
      'operations'      => $operations,
      'title'           => t('Updating data resources translations.'),
      'init_message'    => t('Starting import.'),
      'error_message'   => t('Error importing data resources translations.'),
      'file'            => drupal_get_path('module', 'wow') . '/wow.inc',
    );
    if (isset($finished)) {
      $batch['finished'] = $finished;
    }
    return $batch;
  }
  return FALSE;
}

/**
 * Perform data resource translation import as a batch step.
 *
 * @param $service
 *   The wow data service.
 * @param $resource
 *   The resource to import.
 */
function _wow_batch_import($service, $resource, &$context) {
  // TODO: put as function parameter.
  $key = $resource['data key'];
  $module = $resource['base module'];
  $function = $resource['import callback'];

  $response = wow_http_request($service->region, $resource['remote path'], array('locale' => $service->locale));

  if ($response instanceof WoWHttpStatus) {
    // Do not continue the operation if a status code is returned.
    watchdog_exception($module, new WoWHttpException($response), t('An exception has been thrown during %name data resources import.', array('%name' => $resource['name'])));
    return;
  }

  require_once "modules/$module/$module.resources.inc";

  $report = array();
  if (function_exists($function)) {
    // Call the refresh callback if defined.
    if ($import_result = $function($service, $response->getObject($resource['data key']))) {
      $context['message'] = t('Imported: %name.', array('%name' => $resource['name']));
      $context['results'][$module . ':' . $key] = '';
    }
    else {
      $context['results'][$module . ':' . $key] = 'fail';
    }
  }

}
